// src/client/DDBClient.ts
import {BatchWriteItemCommand, DeleteItemCommand, PutItemCommand, UpdateItemCommand} from "@aws-sdk/client-dynamodb";
import {unmarshall as unmarshall2} from "@aws-sdk/util-dynamodb";

// src/client/DDBReadOnlyClient.ts
import {DynamoDBClient, GetItemCommand, QueryCommand} from "@aws-sdk/client-dynamodb";
import {unmarshall} from "@aws-sdk/util-dynamodb";

// src/client/mapper/mapKey.ts
import {marshall} from "@aws-sdk/util-dynamodb";
var mapKey = (config, partitionKey, sortKey) => {
  return config.sortKeyName && sortKey ? marshall({
    [config.partitionKeyName]: partitionKey,
    [config.sortKeyName]: sortKey
  }) : marshall({
    [config.partitionKeyName]: partitionKey
  });
};

// src/client/mapper/mapGetItemCommand.ts
var mapGetItemCommand = (config, partitionKey, sortKey) => {
  return {
    TableName: config.tableName,
    Key: mapKey(config, partitionKey, sortKey)
  };
};

// src/client/mapper/mapGetItemsCommand.ts
import {marshall as marshall2} from "@aws-sdk/util-dynamodb";
var mapGetItemsCommand = (config, partitionKey, sortKeyPrefix) => {
  const partitionKeyName = String(config.partitionKeyName);
  const sortKeyName = String(config.sortKeyName);
  return {
    TableName: config.tableName,
    IndexName: config.indexName,
    KeyConditionExpression: sortKeyPrefix ? `${partitionKeyName} = :partitionKey and begins_with(${sortKeyName}, :sortKeyPrefix)` : `${partitionKeyName} = :partitionKey`,
    ExpressionAttributeValues: sortKeyPrefix ? marshall2({ ":partitionKey": partitionKey, ":sortKeyPrefix": sortKeyPrefix }) : marshall2({ ":partitionKey": partitionKey })
  };
};

// src/client/mapper/mapGetItemIndexCommand.ts
import {marshall as marshall3} from "@aws-sdk/util-dynamodb";
var mapGetItemIndexCommand = (config, partitionKey, sortKey) => {
  const partitionKeyName = String(config.partitionKeyName);
  const sortKeyName = String(config.sortKeyName);
  return {
    TableName: config.tableName,
    IndexName: config.indexName,
    KeyConditionExpression: sortKey ? `${partitionKeyName} = :partitionKey and ${sortKeyName} = :sortKey` : `${partitionKeyName} = :partitionKey`,
    ExpressionAttributeValues: sortKey ? marshall3({ ":partitionKey": partitionKey, ":sortKey": sortKey }) : marshall3({ ":partitionKey": partitionKey })
  };
};

// src/client/formatter/formatKey.ts
var formatKey = (config, partitionKey, sortKey) => {
  return config.sortKeyName ? `(${String(config.partitionKeyName)}: '${partitionKey}', ${String(config.sortKeyName)}: '${sortKey}')` : `(${String(config.partitionKeyName)}: '${partitionKey}')`;
};

// src/client/DDBReadOnlyClient.ts
class DDBReadOnlyClient {
  config;
  client;
  constructor(config) {
    this.config = config;
    this.client = new DynamoDBClient(config.configuration || {});
  }
  async getItem(partitionKey, sortKey) {
    try {
      this.requireFullPrimaryKey(partitionKey, sortKey);
      if (!this.config.indexName) {
        const response = await this.client.send(new GetItemCommand(mapGetItemCommand(this.config, partitionKey, sortKey)));
        return response?.Item ? unmarshall(response?.Item) : null;
      } else {
        const response = await this.client.send(new QueryCommand(mapGetItemIndexCommand(this.config, partitionKey, sortKey)));
        const items = this.extractItems(response);
        return items.length > 0 ? items[0] : null;
      }
    } catch (err) {
      console.error(formatKey(this.config, partitionKey, sortKey));
      throw new Error(err);
    }
  }
  async getItems(partitionKey, sortKeyPrefix) {
    try {
      const response = await this.client.send(new QueryCommand(mapGetItemsCommand(this.config, partitionKey, sortKeyPrefix)));
      return this.extractItems(response);
    } catch (err) {
      console.error(formatKey(this.config, partitionKey, sortKeyPrefix));
      throw new Error(err);
    }
  }
  extractItems(response) {
    return response?.Items?.map((item) => unmarshall(item)) || [];
  }
  requireFullPrimaryKey(partitionKey, sortKey) {
    if (!partitionKey)
      throw new Error(`Missing partitionKey.`);
    if (this.config.sortKeyName && !sortKey)
      throw new Error(`Missing sortKey.`);
  }
}

// src/client/mapper/mapUpdateItemCommand.ts
import {marshall as marshall4} from "@aws-sdk/util-dynamodb";
var mapUpdateItemCommand = (config, item, createIfNotExists = false) => {
  const partitionKeyName = String(config.partitionKeyName);
  const partitionKey = item[config.partitionKeyName];
  const sortKeyName = String(config.sortKeyName);
  const sortKey = config.sortKeyName ? item[config.sortKeyName] : undefined;
  return {
    TableName: config.tableName,
    Key: mapKey(config, partitionKey, sortKey),
    ConditionExpression: createIfNotExists ? undefined : `#${partitionKeyName} = :${partitionKeyName} and #${sortKeyName} = :${sortKeyName}`,
    ExpressionAttributeValues: marshall4(mapAttributes(config, item, createIfNotExists).reduce((attributeValues, [key, value]) => ({
      ...attributeValues,
      [`:${key}`]: value
    }), {})),
    ExpressionAttributeNames: mapAttributes(config, item, createIfNotExists).reduce((attributeNames, [key]) => ({
      ...attributeNames,
      [`#${key}`]: key
    }), {}),
    UpdateExpression: `SET ${Object.keys(item).filter((key) => key !== config.partitionKeyName && key !== config.sortKeyName).map((key) => `#${key}=:${key}`).join(",")}`,
    ReturnValues: "ALL_NEW"
  };
};
var mapAttributes = (config, item, createIfNotExists = false) => {
  return Object.entries(item).filter(([key]) => !createIfNotExists || createIfNotExists && key !== config.partitionKeyName && key !== config.sortKeyName);
};

// src/client/mapper/mapDeleteItemsCommand.ts
var mapDeleteItemsCommand = (config, items) => {
  return {
    RequestItems: {
      [config.tableName]: items.map(({ partitionKey, sortKey }) => ({
        DeleteRequest: {
          Key: mapKey(config, partitionKey, sortKey)
        }
      }))
    }
  };
};

// src/client/mapper/mapDeleteItemCommand.ts
var mapDeleteItemCommand = (config, partitionKey, sortKey) => {
  return {
    TableName: config.tableName,
    Key: mapKey(config, partitionKey, sortKey)
  };
};

// src/client/mapper/mapCreateItemsCommand.ts
import {marshall as marshall5} from "@aws-sdk/util-dynamodb";
var mapCreateItemsCommand = (config, items) => {
  return {
    RequestItems: {
      [config.tableName]: items.map((item) => {
        if (!item[config.partitionKeyName])
          throw new Error(`Missing partitionKey.`);
        if (config.sortKeyName && !item[config.sortKeyName])
          throw new Error(`Missing sortKey.`);
        return {
          PutRequest: {
            Item: marshall5(item)
          }
        };
      })
    }
  };
};

// src/client/mapper/mapCreateItemCommand.ts
import {marshall as marshall6} from "@aws-sdk/util-dynamodb";
var mapCreateItemCommand = (config, item) => {
  return {
    TableName: config.tableName,
    Item: marshall6(item)
  };
};

// src/client/DDBClient.ts
class DDBClient extends DDBReadOnlyClient {
  constructor(configuration) {
    super(configuration);
  }
  async createItem(item) {
    try {
      const partitionKey = item[this.config.partitionKeyName];
      const sortKey = this.config.sortKeyName ? item[this.config.sortKeyName] : undefined;
      this.requireFullPrimaryKey(partitionKey, sortKey);
      await this.client.send(new PutItemCommand(mapCreateItemCommand(this.config, item)));
      return item;
    } catch (err) {
      console.error("item", JSON.stringify(item));
      throw new Error(err);
    }
  }
  async createItems(items) {
    try {
      await this.client.send(new BatchWriteItemCommand(mapCreateItemsCommand(this.config, items)));
    } catch (err) {
      console.error("items", JSON.stringify(items));
      throw new Error(err);
    }
  }
  async updateItem(item, createIfNotExists = false) {
    const partitionKey = item[this.config.partitionKeyName];
    const sortKey = this.config.sortKeyName ? item[this.config.sortKeyName] : undefined;
    try {
      this.requireFullPrimaryKey(partitionKey, sortKey);
      const response = await this.client.send(new UpdateItemCommand(mapUpdateItemCommand(this.config, item, createIfNotExists)));
      if (!response.Attributes)
        throw new Error("Updated item not returned");
      return unmarshall2(response.Attributes);
    } catch (error) {
      if (error.name === "ConditionalCheckFailedException") {
        throw new Error(`Can't update item as it does not exist: ${formatKey(this.config, partitionKey, sortKey)}`);
      }
      console.error("item", JSON.stringify(item));
      throw new Error(error);
    }
  }
  async deleteItem(partitionKey, sortKey) {
    try {
      this.requireFullPrimaryKey(partitionKey, sortKey);
      await this.client.send(new DeleteItemCommand(mapDeleteItemCommand(this.config, partitionKey, sortKey)));
    } catch (err) {
      console.error(formatKey(this.config, partitionKey, sortKey));
      throw new Error(err);
    }
  }
  async deleteItems(items) {
    try {
      if (items.length === 0)
        return;
      await this.client.send(new BatchWriteItemCommand(mapDeleteItemsCommand(this.config, items)));
    } catch (err) {
      console.error("items", JSON.stringify(items));
      throw new Error(err);
    }
  }
}
export {
  DDBReadOnlyClient,
  DDBClient
};

//# debugId=32CDD223930DA6A364756e2164756e21
